@@ -38,7 +38,7 @@ from prody.proteins import parsePDB, writePDB
 from .editing import reduceModel
 from .plotting import showAtomicLines
 from .signature import ModeEnsemble, saveModeEnsemble
-from prody.utilities import which
+from prody.utilities import which, isListLike
 from . import matchModes
 
 __all__ = ['ESSA']
@@ -496,6 +496,8 @@ class ESSA:
             fea, sco = list(zip(*tmp1))
             ps.append(sco)
         pdbs = parsePDB(l)
+        if not isListLike(pdbs):
+            pdbs = [pdbs]
         chdir('../..')
 
         # ----- # ----- #
